@@ -401,7 +401,8 @@ let generate_apply_funs cont =
                                      J.ENum (float n)),
                              J.ECall (f', params'),
                              J.ECall (J.EVar "caml_call_gen",
-                                      [f'; J.EArr (List.map (fun x -> Some x) params')])))))]) ::
+                                      [f'; J.EArr (List.map (fun x -> Some x) params')])))))],
+          None) ::
        cont)
     funs cont
 
@@ -530,7 +531,7 @@ let _ =
   register_bin_prim "%int_add" `Pure
     (fun cx cy -> Js_simpl.eplus_int cx cy);
   register_bin_prim "%int_sub" `Pure
-    (fun cx cy -> J.EBin (J.Minus, cx, cy));
+    (fun cx cy -> Js_simpl.eminus_int cx cy);
   register_bin_prim "%direct_int_mul" `Pure
     (fun cx cy -> to_int (J.EBin (J.Mul, cx, cy)));
   register_bin_prim "%direct_int_div" `Pure
@@ -613,6 +614,8 @@ let _ =
     (fun cx cy cz -> J.EBin (J.Eq, J.EAccess (cx, cy), cz));
   register_bin_prim "caml_js_get" `Mutable
     (fun cx cy -> J.EAccess (cx, cy));
+  register_bin_prim "caml_js_delete" `Mutable
+    (fun cx cy -> J.EUn(J.Delete, J.EAccess (cx, cy)));
   register_bin_prim "caml_js_equals" `Mutable
     (fun cx cy -> bool (J.EBin (J.EqEq, cx, cy)));
   register_bin_prim "caml_js_instanceof" `Pure
@@ -652,8 +655,8 @@ let rec collect_closures ctx l =
           None
       in
       let cl =
-        J.EFun (fun_name, List.map Var.to_string args,
-                compile_closure ctx cont)
+        J.EFun ((fun_name, List.map Var.to_string args,
+                 compile_closure ctx cont), Some pc)
       in
       let (l', rem') = collect_closures ctx rem in
       ((x, vars, cl) :: l', rem')
@@ -717,13 +720,14 @@ and translate_expr ctx queue x e =
           None
       in
       let cl =
-        J.EFun (fun_name, List.map Var.to_string args,
-                compile_closure ctx cont)
+        J.EFun ((fun_name, List.map Var.to_string args,
+                 compile_closure ctx cont), Some pc)
       in
       let cl =
         if vars = [] then cl else
-        J.ECall (J.EFun (None, vars,
-                         [J.Statement (J.Return_statement (Some cl))]),
+        J.ECall (J.EFun ((None, vars,
+                          [J.Statement (J.Return_statement (Some cl))]),
+                         Some pc),
                  List.map (fun x -> J.EVar x) vars)
       in
       (cl, flush_p, queue)
@@ -800,6 +804,9 @@ and translate_expr ctx queue x e =
           let ((pv, cv), queue) = access_queue queue v in
           (J.EBin (J.Eq, J.EDot (co, f), cv),
            or_p (or_p po pv) mutator_p, queue)
+      | Extern "caml_js_delete", [Pv o; Pc (String f)] ->
+          let ((po, co), queue) = access_queue queue o in
+          (J.EUn(J.Delete, J.EDot (co, f)), or_p po mutator_p, queue)
       | Extern "%object_literal", fields ->
           let rec build_fields l =
             match l with
@@ -811,6 +818,20 @@ and translate_expr ctx queue x e =
                 assert false
           in
           (J.EObj (build_fields fields), const_p, queue)
+      | Extern "caml_js_opt_object", fields ->
+          let rec build_fields queue l =
+            match l with
+              [] ->
+                (const_p, [], queue)
+            | Pc (String nm) :: Pv x :: r ->
+                let ((prop, cx), queue) = access_queue queue x in
+                let (prop', r', queue') = build_fields queue r in
+                (or_p prop prop', (J.PNS nm, cx) :: r', queue)
+            | _ ->
+                assert false
+          in
+          let (prop, fields, queue) = build_fields queue fields in
+          (J.EObj fields, prop, queue)
       | Extern name, l ->
           let name = Primitive.resolve name in
           begin match internal_prim name with
@@ -883,13 +904,14 @@ and translate_closures ctx expr_queue l =
       in
       let cl =
         if vars = [] then cl else
-        J.ECall (J.EFun (None, vars,
-                         [J.Statement (J.Return_statement (Some cl))]),
+        J.ECall (J.EFun ((None, vars,
+                           [J.Statement (J.Return_statement (Some cl))]),
+                          None),
                  List.map (fun x -> J.EVar x) vars)
       in
       let (st, expr_queue) =
         match ctx.Ctx.live.(Var.idx x) with
-          0 -> flush_queue expr_queue flush_p [J.Expression_statement cl]
+          0 -> flush_queue expr_queue flush_p [J.Expression_statement (cl, None)]
         | 1 -> enqueue expr_queue flush_p x cl
         | _ -> flush_queue expr_queue flush_p
                  [J.Variable_statement [Var.to_string x, Some cl]]
@@ -930,18 +952,19 @@ and translate_closures ctx expr_queue l =
             ((Var.to_string tbl,
               Some
                 (J.ECall
-                   (J.EFun (None, vars,
-                            [J.Statement (J.Variable_statement defs);
-                             J.Statement (J.Return_statement (Some arr))]),
+                   (J.EFun ((None, vars,
+                             [J.Statement (J.Variable_statement defs);
+                              J.Statement (J.Return_statement (Some arr))]),
+                            None),
                     List.map (fun x -> J.EVar x) vars))) ::
-             List.rev (fst assgn))
+              List.rev (fst assgn))
         end
       in
       let (st, expr_queue) = flush_queue expr_queue flush_p [statement] in
       let (st', expr_queue) = translate_closures ctx expr_queue rem in
       (st @ st', expr_queue)
 
-and translate_instr ctx expr_queue instr =
+and translate_instr ctx expr_queue pc instr =
   match instr with
     [] ->
       ([], expr_queue)
@@ -949,7 +972,7 @@ and translate_instr ctx expr_queue instr =
       let (l, rem) = collect_closures ctx instr in
       let l = group_closures l in
       let (st, expr_queue) = translate_closures ctx expr_queue l in
-      let (instrs, expr_queue) = translate_instr ctx expr_queue rem in
+      let (instrs, expr_queue) = translate_instr ctx expr_queue pc rem in
       (st @ instrs, expr_queue)
   | i :: rem ->
       let (st, expr_queue) =
@@ -957,7 +980,8 @@ and translate_instr ctx expr_queue instr =
           Let (x, e) ->
             let (ce, prop, expr_queue) = translate_expr ctx expr_queue x e in
             begin match ctx.Ctx.live.(Var.idx x) with
-              0 -> flush_queue expr_queue prop [J.Expression_statement ce]
+              0 -> flush_queue expr_queue prop [J.Expression_statement
+                                                   (ce, Some pc)]
             | 1 -> enqueue expr_queue prop x ce
             | _ -> flush_queue expr_queue prop
                      [J.Variable_statement [Var.to_string x, Some ce]]
@@ -967,23 +991,25 @@ and translate_instr ctx expr_queue instr =
             let ((py, cy), expr_queue) = access_queue expr_queue y in
             flush_queue expr_queue mutator_p
               [J.Expression_statement
-                 (J.EBin (J.Eq, J.EAccess (cx, int (n + 1)), cy))]
+                  ((J.EBin (J.Eq, J.EAccess (cx, int (n + 1)), cy)), Some pc)]
         | Offset_ref (x, n) ->
 (* FIX: may overflow.. *)
             let ((px, cx), expr_queue) = access_queue expr_queue x in
             flush_queue expr_queue mutator_p
               [J.Expression_statement
-                 (J.EBin (J.PlusEq, (J.EAccess (cx, J.ENum 1.)), int n))]
+                  ((J.EBin (J.PlusEq, (J.EAccess (cx, J.ENum 1.)), int n)),
+                  Some pc)]
         | Array_set (x, y, z) ->
             let ((px, cx), expr_queue) = access_queue expr_queue x in
             let ((py, cy), expr_queue) = access_queue expr_queue y in
             let ((pz, cz), expr_queue) = access_queue expr_queue z in
             flush_queue expr_queue mutator_p
               [J.Expression_statement
-                 (J.EBin (J.Eq, J.EAccess (cx, J.EBin(J.Plus, cy, one)),
-                          cz))]
+                  ((J.EBin (J.Eq, J.EAccess (cx, J.EBin(J.Plus, cy, one)),
+                            cz)),
+                   Some pc)]
       in
-      let (instrs, expr_queue) = translate_instr ctx expr_queue rem in
+      let (instrs, expr_queue) = translate_instr ctx expr_queue pc rem in
       (st @ instrs, expr_queue)
 
 and compile_block st queue pc frontier interm =
@@ -1030,7 +1056,7 @@ AddrSet.iter (fun pc -> Format.eprintf " %d" pc) frontier;
 Format.eprintf ">>@ ";
 *)
   let block = AddrMap.find pc st.blocks in
-  let (seq, queue) = translate_instr st.ctx queue block.body in
+  let (seq, queue) = translate_instr st.ctx queue pc block.body in
   let body =
     seq @
     match block.branch with
@@ -1068,7 +1094,8 @@ Format.eprintf "===== %d ===== (%b)@." pc3 limit_body;
           (J.Try_statement (Js_simpl.statement_list body,
                             Some (Var.to_string x,
                                   Js_simpl.statement_list handler),
-                            None) ::
+                            None,
+                            Some pc) ::
            if AddrSet.is_empty inner_frontier then [] else begin
              let pc = AddrSet.choose inner_frontier in
 (*
@@ -1162,7 +1189,8 @@ res
             end else begin
               if debug () then Format.eprintf "}@]";
               body
-            end))
+            end),
+        Some pc)
     in
     if label = -1 then
       [st]
@@ -1451,12 +1479,12 @@ let compile_program standalone ctx pc =
   Primitive.list_used ();
 *)
   if standalone then
-    let f = J.EFun (None, [], generate_apply_funs res) in
-    [J.Statement (J.Expression_statement (J.ECall (f, [])))]
+    let f = J.EFun ((None, [], generate_apply_funs res), None) in
+    [J.Statement (J.Expression_statement ((J.ECall (f, [])), Some pc))]
   else
-    let f = J.EFun (None, [Var.to_string (Var.fresh ())],
-                    generate_apply_funs res) in
-    [J.Statement (J.Expression_statement f)]
+    let f = J.EFun ((None, [Var.to_string (Var.fresh ())],
+                     generate_apply_funs res), None) in
+    [J.Statement (J.Expression_statement (f, Some pc))]
 
 (**********************)
 
@@ -1466,7 +1494,7 @@ let list_missing l =
     List.iter (fun nm -> Format.eprintf "  %s@." nm) l
   end
 
-let f ch ?(standalone=true) ((pc, blocks, _) as p) live_vars =
+let f ch ?(standalone=true) ?linkall ((pc, blocks, _) as p) dl live_vars =
   let mutated_vars = Freevars.f p in
   let t' = Util.Timer.make () in
   let ctx = Ctx.initial blocks live_vars mutated_vars in
@@ -1474,13 +1502,13 @@ let f ch ?(standalone=true) ((pc, blocks, _) as p) live_vars =
   if !compact then Pretty_print.set_compact ch true;
   if standalone then begin
     Pretty_print.string ch
-      "// This program was compiled from OCaml by js_of_ocaml 1.0";
+      "// This program was compiled from OCaml by js_of_ocaml 1.4";
     Pretty_print.newline ch;
-    let missing = Linker.resolve_deps !compact ch (Primitive.get_used ()) in
+    let missing = Linker.resolve_deps ?linkall !compact ch (Primitive.get_used ()) in
     list_missing missing
   end;
   Hashtbl.clear add_names;
-  let res = Js_output.program ch p in
+  let res = Js_output.program ch p dl in
   if times () then Format.eprintf "  code gen.: %a@." Util.Timer.print t';
   res
 
