@@ -89,7 +89,8 @@ let rec formal_parameter_list f l =
 
 let op_prec op =
   match op with
-    Eq | StarEq | SlashEq | ModEq | PlusEq | MinusEq -> 1, 13, 1
+    Eq | StarEq | SlashEq | ModEq | PlusEq | MinusEq
+  | LslEq | AsrEq | LsrEq | BandEq | BxorEq | BorEq -> 1, 13, 1
 (*
   | Or -> 3, 3, 4
   | And -> 4, 4, 5
@@ -125,6 +126,12 @@ let op_str op =
   | NotEq   -> "!="
   | EqEqEq  -> "==="
   | NotEqEq -> "!=="
+  | LslEq   -> "<<="
+  | AsrEq   -> ">>="
+  | LsrEq   -> ">>>="
+  | BandEq  -> "&="
+  | BxorEq  -> "^="
+  | BorEq   -> "|="
   | Lt      -> "<"
   | Le      -> "<="
   | Lsl     -> "<<"
@@ -142,6 +149,8 @@ let unop_str op =
     Not -> "!"
   | Neg -> "-"
   | Pl  -> "+"
+  | Bnot -> "~"
+  | IncrA | IncrB | DecrA | DecrB
   | Typeof | Delete -> assert false
 
 (*XXX May need to be updated... *)
@@ -268,6 +277,19 @@ let rec expression l f e =
         PP.string f "NaN"
       else begin
         let s =
+          let vint = int_of_float v in
+          (* compiler 1000 into 1e3 *)
+          if float_of_int vint = v
+          then
+            let rec div n i =
+              if n <> 0 && n mod 10 = 0
+              then div (n/10) (succ i)
+              else
+              if i > 2
+              then Printf.sprintf "%de%d" n i
+              else string_of_int vint in
+            div vint 0
+          else
           let s1 = Printf.sprintf "%.12g" v in
           if v = float_of_string s1 then s1 else
           let s2 = Printf.sprintf "%.15g" v in
@@ -301,6 +323,16 @@ let rec expression l f e =
       expression 13 f e;
       PP.end_group f;
       if l > 13 then begin PP.string f ")"; PP.end_group f end
+  | EUn ((IncrA | DecrA | IncrB | DecrB) as op,e) ->
+    if l > 13 then begin PP.start_group f 1; PP.string f "(" end;
+    if op = IncrA || op = DecrA
+    then expression 13 f e;
+    if op = IncrA || op = IncrB
+    then PP.string f "++"
+    else PP.string f "--";
+    if op = IncrB || op = DecrB
+    then expression 13 f e;
+    if l > 13 then begin PP.string f ")"; PP.end_group f end
   | EUn (op, e) ->
       if l > 13 then begin PP.start_group f 1; PP.string f "(" end;
       PP.string f (unop_str op);
@@ -511,6 +543,7 @@ and statement f s =
           PP.string f ";";
           PP.end_group f
       end
+  | Expression_statement (EVar _, pc)-> ()
   | Expression_statement (e, pc) ->
       (* Parentheses are required when the expression
          starts syntactically with "{" or "function" *)
@@ -854,4 +887,3 @@ let statement f s dl =
 let program f se dl =
   debug_info := Some dl;
   PP.start_group f 0; source_elements f se; PP.end_group f; PP.newline f
-
