@@ -40,26 +40,38 @@ let rec tail_call x f l =
   | i :: rem ->
       tail_call x f rem
 
-let rewrite_block (f, f_params, f_pc, args) pc blocks =
-(*Format.eprintf "%d@." pc;*)
-  let block = AddrMap.find pc blocks in
+let rec return ?(subst=VarMap.empty) block blocks =
   match block.branch with
-    Return x ->
-      begin match tail_call x f block.body with
-        Some f_args ->
-          let m = Subst.build_mapping f_params f_args in
-          AddrMap.add pc
-            { params = block.params;
-              handler = block.handler;
-              body = remove_last block.body;
-              branch =
-                Branch
-                  (f_pc, List.map (fun x -> VarMap.find x m) args) }
+    | Return x -> Some (Subst.from_map subst x)
+    | Branch (pc,l) ->
+      let block = AddrMap.find pc blocks in
+      if block.body = []
+      then
+        let subst = Subst.build_mapping block.params l in
+        return ~subst block blocks
+      else None
+    | _ -> None
+
+let rewrite_block (f, f_params, f_pc, args) pc blocks =
+  (*Format.eprintf "%d@." pc;*)
+  let block = AddrMap.find pc blocks in
+  match return block blocks with
+    | Some x ->
+        begin match tail_call x f block.body with
+            Some f_args when List.length f_params = List.length f_args ->
+            let m = Subst.build_mapping f_params f_args in
+            AddrMap.add pc
+              { params = block.params;
+                handler = block.handler;
+                body = remove_last block.body;
+                branch =
+                  Branch
+                    (f_pc, List.map (fun x -> VarMap.find x m) args) }
+              blocks
+          | _ ->
             blocks
-      | _ ->
-          blocks
-      end
-  | _ ->
+        end
+    | _ ->
       blocks
 
 let (>>) x f = f x
@@ -102,12 +114,12 @@ let f ((pc, blocks, free_pc) as p) =
     fold_closures p
       (fun f params (pc, args) blocks ->
          match f with
-             Some f ->
-               let (_, blocks) =
-                 traverse (f, params, pc, args) pc AddrSet.empty blocks in
-                 blocks
-           | None ->
-               blocks)
+             Some f when List.length params = List.length args ->
+             let (_, blocks) =
+               traverse (f, params, pc, args) pc AddrSet.empty blocks in
+             blocks
+           | _ ->
+             blocks)
       blocks
   in
   if times () then Format.eprintf "  tail calls: %a@." Util.Timer.print t;
