@@ -218,7 +218,7 @@ and touch = object
 end
 
 and dragEvent = object
-  inherit event
+  inherit mouseEvent
   method dataTransfer : dataTransfer t readonly_prop
 end
 
@@ -248,6 +248,7 @@ and eventTarget = object ('self)
   method onkeypress : ('self t, keyboardEvent t) event_listener writeonly_prop
   method onkeydown : ('self t, keyboardEvent t) event_listener writeonly_prop
   method onkeyup : ('self t, keyboardEvent t) event_listener writeonly_prop
+  method onscroll : ('self t, event t) event_listener writeonly_prop
   method ondragstart : ('self t, dragEvent t) event_listener writeonly_prop
   method ondragend : ('self t, dragEvent t) event_listener writeonly_prop
   method ondragenter : ('self t, dragEvent t) event_listener writeonly_prop
@@ -489,10 +490,12 @@ class type inputElement = object ('self)
   method select : unit meth
   method click : unit meth
   method files : File.fileList t optdef readonly_prop
-
+  method placeholder : js_string t writeonly_prop (* Not supported by IE 9 *)
   method onselect : ('self t, event t) event_listener prop
   method onchange : ('self t, event t) event_listener prop
   method oninput : ('self t, event t) event_listener prop
+  method onblur : ('self t, event t) event_listener prop
+  method onfocus : ('self t, event t) event_listener prop
 end
 
 class type textAreaElement = object ('self)
@@ -511,10 +514,13 @@ class type textAreaElement = object ('self)
   method blur : unit meth
   method focus : unit meth
   method select : unit meth
+  method placeholder : js_string t writeonly_prop (* Not supported by IE 9 *)
 
   method onselect : ('self t, event t) event_listener prop
   method onchange : ('self t, event t) event_listener prop
   method oninput : ('self t, event t) event_listener prop
+  method onblur : ('self t, event t) event_listener prop
+  method onfocus : ('self t, event t) event_listener prop
 end
 
 class type buttonElement = object
@@ -665,6 +671,7 @@ class type scriptElement = object
   method defer : bool t prop
   method src : js_string t prop
   method _type : js_string t prop
+  method async : bool t prop
 end
 
 class type tableCellElement = object
@@ -885,9 +892,11 @@ external pixel_set : canvasPixelArray t -> int -> int -> unit = "caml_js_set"
 class type document = object
   inherit [element] Dom.document
   inherit nodeSelector
+  inherit eventTarget
+
   method title : js_string t prop
   method referrer : js_string t readonly_prop
-  method domain : js_string t readonly_prop
+  method domain : js_string t prop
   method _URL : js_string t readonly_prop
   method head : headElement t prop
   method body : bodyElement t prop
@@ -955,7 +964,7 @@ class type navigator = object
   method appName : js_string t readonly_prop
   method appVersion : js_string t readonly_prop
   method cookieEnabled : bool t readonly_prop
-  method online : bool t readonly_prop
+  method onLine : bool t readonly_prop
   method platform : js_string t readonly_prop
   method userAgent : js_string t readonly_prop
   method language : js_string t optdef readonly_prop
@@ -969,17 +978,39 @@ class type screen = object
   method availHeight : int readonly_prop
 end
 
+class type applicationCache = object
+  method status : int readonly_prop
+
+  method update : unit meth
+  method abort : unit meth
+  method swapCache : unit meth
+
+  method onchecking : (applicationCache t, event t) event_listener prop
+  method onerror : (applicationCache t, event t) event_listener prop
+  method onnoupdate : (applicationCache t, event t) event_listener prop
+  method ondownloading : (applicationCache t, event t) event_listener prop
+  method onprogress : (applicationCache t, event t) event_listener prop
+  method onupdateready : (applicationCache t, event t) event_listener prop
+  method oncached : (applicationCache t, event t) event_listener prop
+  method onobsolete : (applicationCache t, event t) event_listener prop
+
+  inherit eventTarget
+end
+
 type interval_id
 type timeout_id
 
 (** Specification of window objects *)
 class type window = object
+  inherit eventTarget
+
   method document : document t readonly_prop
+  method applicationCache : applicationCache t readonly_prop
   method name : js_string t prop
   method location : location t readonly_prop
   method history : history t readonly_prop
   method undoManager : undoManager t readonly_prop
-  method navigator : navigator t
+  method navigator : navigator t readonly_prop
   method getSelection : selection t meth
   method close : unit meth
   method closed : bool t readonly_prop
@@ -987,7 +1018,6 @@ class type window = object
   method focus : unit meth
   method blur : unit meth
   method scroll : int -> int -> unit meth
-  method screen : screen t readonly_prop
 
   method sessionStorage : storage t optdef readonly_prop
   method localStorage : storage t optdef readonly_prop
@@ -999,7 +1029,7 @@ class type window = object
   method open_ : js_string t -> js_string t -> js_string t opt -> window t meth
   method alert : js_string t -> unit meth
   method confirm : js_string t -> bool t meth
-  method prompt : js_string t -> js_string t -> js_string t meth
+  method prompt : js_string t -> js_string t -> js_string t opt meth
   method print : unit meth
 
   method setInterval : (unit -> unit) Js.callback -> float -> interval_id meth
@@ -1008,14 +1038,22 @@ class type window = object
   method setTimeout : (unit -> unit) Js.callback -> float -> timeout_id meth
   method clearTimeout : timeout_id -> unit meth
 
+  method screen : screen t readonly_prop
+  method innerWidth : int optdef readonly_prop
+  method innerHeight : int optdef readonly_prop
+  method outerWidth : int optdef readonly_prop
+  method outerHeight : int optdef readonly_prop
+
   method onload : (window t, event t) event_listener prop
   method onbeforeunload : (window t, event t) event_listener prop
   method onblur : (window t, event t) event_listener prop
   method onfocus : (window t, event t) event_listener prop
   method onresize : (window t, event t) event_listener prop
-  method onscroll : (window t, event t) event_listener prop
   method onpopstate : (window t, popStateEvent t) event_listener prop
   method onhashchange : (window t, hashChangeEvent t) event_listener prop
+
+  method ononline : (window t, event t) event_listener writeonly_prop
+  method onoffline : (window t, event t) event_listener writeonly_prop
 end
 
 val window : window t
@@ -1112,6 +1150,30 @@ module Event : sig
   val hashchange : hashChangeEvent t typ
   val change : event t typ
   val input : event t typ
+  val submit : event t typ
+  val scroll : event t typ
+  val focus : event t typ
+  val blur : event t typ
+  val load : event t typ
+  val beforeunload : event t typ
+  val resize : event t typ
+  val popstate : event t typ
+  val hashchange : event t typ
+  val error : event t typ
+  val abort : event t typ
+  val select : event t typ
+
+  val online : event t typ
+  val offline : event t typ
+
+  val checking : event t typ
+  val error : event t typ
+  val noupdate : event t typ
+  val downloading : event t typ
+  val progress : event t typ
+  val updateready : event t typ
+  val cached : event t typ
+  val obsolete : event t typ
 
   val make : string -> 'a typ
 end
@@ -1408,3 +1470,4 @@ end
 (**/**)
 
 val onIE : bool
+val hasPushState : unit -> bool
