@@ -84,6 +84,10 @@ function caml_raise_zero_divide () {
   caml_raise_constant(caml_global_data[6]);
 }
 
+//Provides: caml_raise_not_found
+//Requires: caml_raise_constant, caml_global_data
+function caml_raise_not_found () { caml_raise_constant(caml_global_data[7]); }
+
 //Provides: caml_update_dummy
 function caml_update_dummy (x, y) {
   if( typeof y==="function" ) { x.fun = y; return 0; }
@@ -275,7 +279,7 @@ function caml_parse_digit(c) {
   return -1;
 }
 
-//Provides: caml_int_of_string
+//Provides: caml_int_of_string mutable
 //Requires: caml_parse_sign_and_base, caml_parse_digit, MlString, caml_failwith
 function caml_int_of_string (s) {
   var r = caml_parse_sign_and_base (s);
@@ -519,6 +523,141 @@ function caml_hash_univ_param (count, limit, obj) {
   return hash_accu & 0x3FFFFFFF;
 }
 
+//Provides: caml_hash mutable
+//Requires: MlString, caml_int64_bits_of_float
+var caml_hash =
+function () {
+  var HASH_QUEUE_SIZE = 256;
+  function ROTL32(x,n) { return ((x << n) | (x >>> (32-n))); }
+  function MIX(h,d) {
+    d = caml_mul(d, 0xcc9e2d51);
+    d = ROTL32(d, 15);
+    d = caml_mul(d, 0x1b873593);
+    h ^= d;
+    h = ROTL32(h, 13);
+    return ((((h * 5)|0) + 0xe6546b64)|0);
+  }
+  function FINAL_MIX(h) {
+    h ^= h >>> 16;
+    h = caml_mul (h, 0x85ebca6b);
+    h ^= h >>> 13;
+    h = caml_mul (h, 0xc2b2ae35);
+    h ^= h >>> 16;
+    return h;
+  }
+  function caml_hash_mix_int64 (h, v) {
+    var lo = v[1] | (v[2] << 24);
+    var hi = (v[2] >>> 8) | (v[3] << 16);
+    h = MIX(h, lo);
+    h = MIX(h, hi);
+    return h;
+  }
+  function caml_hash_mix_int64_2 (h, v) {
+    var lo = v[1] | (v[2] << 24);
+    var hi = (v[2] >>> 8) | (v[3] << 16);
+    h = MIX(h, hi ^ lo);
+    return h;
+  }
+  function caml_hash_mix_string_str(h, s) {
+    var len = s.length, i, w;
+    for (i = 0; i + 4 <= len; i += 4) {
+      w = s.charCodeAt(i)
+          | (s.charCodeAt(i+1) << 8)
+          | (s.charCodeAt(i+2) << 16)
+          | (s.charCodeAt(i+3) << 24);
+      h = MIX(h, w);
+    }
+    w = 0;
+    switch (len & 3) {
+    case 3: w  = s.charCodeAt(i+2) << 16;
+    case 2: w |= s.charCodeAt(i+1) << 8;
+    case 1: w |= s.charCodeAt(i);
+            h = MIX(h, w);
+    default:
+    }
+    h ^= len;
+    return h;
+  }
+  function caml_hash_mix_string_arr(h, s) {
+    var len = s.length, i, w;
+    for (i = 0; i + 4 <= len; i += 4) {
+      w = s[i]
+          | (s[i+1] << 8)
+          | (s[i+2] << 16)
+          | (s[i+3] << 24);
+      h = MIX(h, w);
+    }
+    w = 0;
+    switch (len & 3) {
+    case 3: w  = s[i+2] << 16;
+    case 2: w |= s[i+1] << 8;
+    case 1: w |= s[i];
+            h = MIX(h, w);
+    default:
+    }
+    h ^= len;
+    return h;
+  }
+  return function (count, limit, seed, obj) {
+    var queue, rd, wr, sz, num, h, v, i, len;
+    sz = limit;
+    if (sz < 0 || sz > HASH_QUEUE_SIZE) sz = HASH_QUEUE_SIZE;
+    num = count;
+    h = seed;
+    queue = [obj]; rd = 0; wr = 1;
+    while (rd < wr && num > 0) {
+      v = queue[rd++];
+      if (v instanceof Array && v[0] === (v[0]|0)) {
+        switch (v[0]) {
+        case 248:
+          // Object
+          h = MIX(h, v[2]);
+          num--;
+          break;
+        case 250:
+          // Forward
+          queue[--rd] = v[1];
+          break;
+        case 255:
+          // Int64
+          h = caml_hash_mix_int64_2 (h, v);
+          num --;
+          break;
+        default:
+          var tag = ((v.length - 1) << 10) | v[0];
+          h = MIX(h, tag);
+          for (i = 1, len = v.length; i < len; i++) {
+            if (wr >= sz) break;
+            queue[wr++] = v[i];
+          }
+          break;
+        }
+      } else if (v instanceof MlString) {
+        var a = v.array;
+        if (a) {
+          h = caml_hash_mix_string_arr(h, a);
+        } else {
+          var b = v.getFullBytes ();
+          h = caml_hash_mix_string_str(h, b);
+        }
+        num--;
+        break;
+      } else if (v === (v|0)) {
+        // Integer
+        h = MIX(h, v+v+1);
+        num--;
+      } else if (v === +v) {
+        // Float
+        h = caml_hash_mix_int64(h, caml_int64_bits_of_float (v));
+        num--;
+        break;
+      }
+    }
+    h = FINAL_MIX(h);
+    return h & 0x3FFFFFFF;
+  }
+} ();
+
 ///////////// Sys
 //Provides: caml_sys_time mutable
 var caml_initial_time = new Date() * 0.001;
@@ -526,11 +665,54 @@ function caml_sys_time () { return new Date() * 0.001 - caml_initial_time; }
 //Provides: caml_sys_get_config const
 //Requires: MlString
 function caml_sys_get_config () {
-  return [0, new MlWrappedString("Unix"), 32];
+  return [0, new MlWrappedString("Unix"), 32, 0];
 }
 //Provides: caml_sys_random_seed mutable
+//The function needs to return an array since OCaml 4.0...
 function caml_sys_random_seed () {
-  return new Date()^0xffffffff*Math.random();
+  var x = new Date()^0xffffffff*Math.random();
+  return {valueOf:function(){return x;},0:0,1:x,length:2};
+}
+//Provides: caml_sys_const_big_endian const
+function caml_sys_const_big_endian () { return 0; }
+//Provides: caml_sys_const_word_size const
+function caml_sys_const_word_size () { return 32; }
+//Provides: caml_sys_const_ostype_cygwin const
+function caml_sys_const_ostype_cygwin () { return 0; }
+//Provides: caml_sys_const_ostype_unix const
+function caml_sys_const_ostype_unix () { return 1; }
+//Provides: caml_sys_const_ostype_win32 const
+function caml_sys_const_ostype_win32 () { return 0; }
+
+///////////// Array
+//Provides: caml_array_sub mutable
+function caml_array_sub (a, i, len) {
+  return [0].concat(a.slice(i+1, i+1+len));
+}
+
+//Provides: caml_array_append mutable
+function caml_array_append(a1, a2) {
+  return a1.concat(a2.slice(1));
+}
+
+//Provides: caml_array_concat mutable
+function caml_array_concat(l) {
+  var a = [0];
+  while (l != 0) {
+    var b = l[1];
+    for (var i = 1; i < b.length; i++) a.push(b[i]);
+    l = l[2];
+  }
+  return a;
+}
+
+//Provides: caml_array_blit
+function caml_array_blit(a1, i1, a2, i2, len) {
+  if (i2 <= i1) {
+    for (var j = 1; j <= len; j++) a2[i2 + j] = a1[i1 + j];
+  } else {
+    for (var j = len; j >= 1; j--) a2[i2 + j] = a1[i1 + j];
+  }
 }
 
 ///////////// CamlinternalOO
@@ -576,3 +758,16 @@ function caml_get_exception_backtrace () {
   caml_invalid_argument
     ("Primitive 'caml_get_exception_backtrace' not implemented");
 }
+//Provides: caml_get_exception_raw_backtrace const
+function caml_get_exception_raw_backtrace () {
+  caml_invalid_argument
+    ("Primitive 'caml_get_exception_raw_backtrace' not implemented");
+}
+//Provides: caml_convert_raw_backtrace const
+function caml_convert_raw_backtrace () {
+  caml_invalid_argument
+    ("Primitive 'caml_convert_raw_backtrace' not implemented");
+}
+//Provides: caml_sys_getenv
+//Requires: caml_raise_not_found
+function caml_sys_getenv () { caml_raise_not_found (); }
